"""The mark commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MARK {NEXT|PREVious}
    - MARK:CREATE {CH<x>|MATH|B<x>|REF<x>|COLUMN|DIGItal}
    - MARK:DELEte {CH<x>|MATH|B<x>|REF<x>|COLUMN|DIGItal}
    - MARK:FREE?
    - MARK:SELected:END?
    - MARK:SELected:FOCUS?
    - MARK:SELected:MARKSINCOLumn?
    - MARK:SELected:OWNer?
    - MARK:SELected:SOURCE?
    - MARK:SELected:STARt?
    - MARK:SELected:STATe?
    - MARK:SELected:ZOOm:POSition?
    - MARK:TOTal?
    - MARK?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MarkTotal(SCPICmdRead):
    """The ``MARK:TOTal`` command.

    Description:
        - Returns how many marks are currently in use. There can be a total of 1,024 marks returned.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:TOTal?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:TOTal?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:TOTal?
        ```
    """


class MarkSelectedZoomPosition(SCPICmdRead):
    """The ``MARK:SELected:ZOOm:POSition`` command.

    Description:
        - Returns the position of the selected mark, 0 to 100% of the zoom overview window.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:ZOOm:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:ZOOm:POSition?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELected:ZOOm:POSition?
        ```
    """


class MarkSelectedZoom(SCPICmdRead):
    """The ``MARK:SELected:ZOOm`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:ZOOm?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:ZOOm?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.position``: The ``MARK:SELected:ZOOm:POSition`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._position = MarkSelectedZoomPosition(device, f"{self._cmd_syntax}:POSition")

    @property
    def position(self) -> MarkSelectedZoomPosition:
        """Return the ``MARK:SELected:ZOOm:POSition`` command.

        Description:
            - Returns the position of the selected mark, 0 to 100% of the zoom overview window.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:ZOOm:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:ZOOm:POSition?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELected:ZOOm:POSition?
            ```
        """
        return self._position


class MarkSelectedState(SCPICmdRead):
    """The ``MARK:SELected:STATe`` command.

    Description:
        - Returns the on or off state of the selected mark. The selected mark is at or near the
          center of the screen. If you press the front-panel Set/Clear button, this mark will
          disappear.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:STATe?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELected:STATe?
        ```
    """


class MarkSelectedStart(SCPICmdRead):
    """The ``MARK:SELected:STARt`` command.

    Description:
        - Returns the starting point of the selected mark, 0 to 100% of the waveform.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:STARt?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:STARt?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELected:STARt?
        ```
    """


class MarkSelectedSource(SCPICmdRead):
    """The ``MARK:SELected:SOURCE`` command.

    Description:
        - Returns the source waveform for the selected mark.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:SOURCE?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:SOURCE?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELected:SOURCE?
        ```
    """


class MarkSelectedOwner(SCPICmdRead):
    """The ``MARK:SELected:OWNer`` command.

    Description:
        - Returns the owner of the selected mark.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:OWNer?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:OWNer?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELected:OWNer?
        ```
    """


class MarkSelectedMarksincolumn(SCPICmdRead):
    """The ``MARK:SELected:MARKSINCOLumn`` command.

    Description:
        - Returns the number of marks in the current zoom pixel column.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:MARKSINCOLumn?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:MARKSINCOLumn?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELected:MARKSINCOLumn?
        ```
    """


class MarkSelectedFocus(SCPICmdRead):
    """The ``MARK:SELected:FOCUS`` command.

    Description:
        - Returns the focus of the selected mark, 0 to 100% of the waveform.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:FOCUS?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:FOCUS?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELected:FOCUS?
        ```
    """


class MarkSelectedEnd(SCPICmdRead):
    """The ``MARK:SELected:END`` command.

    Description:
        - Returns the end of the selected mark, 0 to 100% of the waveform.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected:END?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected:END?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELected:END?
        ```
    """


#  pylint: disable=too-many-instance-attributes
class MarkSelected(SCPICmdRead):
    """The ``MARK:SELected`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELected?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELected?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.end``: The ``MARK:SELected:END`` command.
        - ``.focus``: The ``MARK:SELected:FOCUS`` command.
        - ``.marksincolumn``: The ``MARK:SELected:MARKSINCOLumn`` command.
        - ``.owner``: The ``MARK:SELected:OWNer`` command.
        - ``.source``: The ``MARK:SELected:SOURCE`` command.
        - ``.start``: The ``MARK:SELected:STARt`` command.
        - ``.state``: The ``MARK:SELected:STATe`` command.
        - ``.zoom``: The ``MARK:SELected:ZOOm`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._end = MarkSelectedEnd(device, f"{self._cmd_syntax}:END")
        self._focus = MarkSelectedFocus(device, f"{self._cmd_syntax}:FOCUS")
        self._marksincolumn = MarkSelectedMarksincolumn(device, f"{self._cmd_syntax}:MARKSINCOLumn")
        self._owner = MarkSelectedOwner(device, f"{self._cmd_syntax}:OWNer")
        self._source = MarkSelectedSource(device, f"{self._cmd_syntax}:SOURCE")
        self._start = MarkSelectedStart(device, f"{self._cmd_syntax}:STARt")
        self._state = MarkSelectedState(device, f"{self._cmd_syntax}:STATe")
        self._zoom = MarkSelectedZoom(device, f"{self._cmd_syntax}:ZOOm")

    @property
    def end(self) -> MarkSelectedEnd:
        """Return the ``MARK:SELected:END`` command.

        Description:
            - Returns the end of the selected mark, 0 to 100% of the waveform.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:END?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:END?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELected:END?
            ```
        """
        return self._end

    @property
    def focus(self) -> MarkSelectedFocus:
        """Return the ``MARK:SELected:FOCUS`` command.

        Description:
            - Returns the focus of the selected mark, 0 to 100% of the waveform.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:FOCUS?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:FOCUS?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELected:FOCUS?
            ```
        """
        return self._focus

    @property
    def marksincolumn(self) -> MarkSelectedMarksincolumn:
        """Return the ``MARK:SELected:MARKSINCOLumn`` command.

        Description:
            - Returns the number of marks in the current zoom pixel column.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:MARKSINCOLumn?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:MARKSINCOLumn?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELected:MARKSINCOLumn?
            ```
        """
        return self._marksincolumn

    @property
    def owner(self) -> MarkSelectedOwner:
        """Return the ``MARK:SELected:OWNer`` command.

        Description:
            - Returns the owner of the selected mark.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:OWNer?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:OWNer?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELected:OWNer?
            ```
        """
        return self._owner

    @property
    def source(self) -> MarkSelectedSource:
        """Return the ``MARK:SELected:SOURCE`` command.

        Description:
            - Returns the source waveform for the selected mark.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:SOURCE?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:SOURCE?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELected:SOURCE?
            ```
        """
        return self._source

    @property
    def start(self) -> MarkSelectedStart:
        """Return the ``MARK:SELected:STARt`` command.

        Description:
            - Returns the starting point of the selected mark, 0 to 100% of the waveform.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:STARt?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:STARt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELected:STARt?
            ```
        """
        return self._start

    @property
    def state(self) -> MarkSelectedState:
        """Return the ``MARK:SELected:STATe`` command.

        Description:
            - Returns the on or off state of the selected mark. The selected mark is at or near the
              center of the screen. If you press the front-panel Set/Clear button, this mark will
              disappear.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:STATe?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELected:STATe?
            ```
        """
        return self._state

    @property
    def zoom(self) -> MarkSelectedZoom:
        """Return the ``MARK:SELected:ZOOm`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected:ZOOm?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected:ZOOm?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.position``: The ``MARK:SELected:ZOOm:POSition`` command.
        """
        return self._zoom


class MarkFree(SCPICmdRead):
    """The ``MARK:FREE`` command.

    Description:
        - Returns how many marks are available for use. There can be a total of 1,024 marks
          returned.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:FREE?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:FREE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:FREE?
        ```
    """


class MarkDelete(SCPICmdWrite):
    """The ``MARK:DELEte`` command.

    Description:
        - Deletes a mark on a particular waveform, all waveforms in a column, the selected mark, or
          all marks.

    Usage:
        - Using the ``.write(value)`` method will send the ``MARK:DELEte value`` command.

    SCPI Syntax:
        ```
        - MARK:DELEte {CH<x>|MATH|B<x>|REF<x>|COLUMN|DIGItal}
        ```

    Info:
        - ``CH<x>`` deletes the mark on a channel waveform, where <x> is the channel number.
        - ``MATH`` deletes the mark on the math waveform.
        - ``B<x>`` deletes the mark on a bus waveform, where <x> is the bus number.
        - ``REF<x>`` deletes the mark on a reference waveform, where <x> is the reference waveform
          number.
        - ``COLUMN`` deletes marks on all waveforms in the current zoom pixel column.
        - ``DIGItal`` deletes all marks on all digital channels.
    """


class MarkCreate(SCPICmdWrite):
    """The ``MARK:CREATE`` command.

    Description:
        - Creates a mark on a specified waveform or all waveforms in a column.

    Usage:
        - Using the ``.write(value)`` method will send the ``MARK:CREATE value`` command.

    SCPI Syntax:
        ```
        - MARK:CREATE {CH<x>|MATH|B<x>|REF<x>|COLUMN|DIGItal}
        ```

    Info:
        - ``CH<x>`` creates the mark on a channel waveform, where <x> is the channel number.
        - ``MATH`` creates the mark on the math waveform.
        - ``B<x>`` creates the mark on a bus waveform, where <x> is the bus number.
        - ``REF<x>`` creates the mark on a reference waveform, where <x> is the reference waveform
          number.
        - ``COLUMN`` creates marks on all waveforms in the current zoom pixel column.
        - ``DIGItal`` creates marks on a digital waveform. (An error will result if no digital
          channel is turned on.).
    """


class Mark(SCPICmdWrite, SCPICmdRead):
    """The ``MARK`` command.

    Description:
        - Moves to the next or previous reference mark on the waveform. Returns the current mark
          position.

    Usage:
        - Using the ``.query()`` method will send the ``MARK?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MARK value`` command.

    SCPI Syntax:
        ```
        - MARK {NEXT|PREVious}
        - MARK?
        ```

    Info:
        - ``NEXT`` moves to the next reference mark on the right.
        - ``PREVious`` moves to the next reference mark on the left.

    Properties:
        - ``.create``: The ``MARK:CREATE`` command.
        - ``.delete``: The ``MARK:DELEte`` command.
        - ``.free``: The ``MARK:FREE`` command.
        - ``.selected``: The ``MARK:SELected`` command tree.
        - ``.total``: The ``MARK:TOTal`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "MARK") -> None:
        super().__init__(device, cmd_syntax)
        self._create = MarkCreate(device, f"{self._cmd_syntax}:CREATE")
        self._delete = MarkDelete(device, f"{self._cmd_syntax}:DELEte")
        self._free = MarkFree(device, f"{self._cmd_syntax}:FREE")
        self._selected = MarkSelected(device, f"{self._cmd_syntax}:SELected")
        self._total = MarkTotal(device, f"{self._cmd_syntax}:TOTal")

    @property
    def create(self) -> MarkCreate:
        """Return the ``MARK:CREATE`` command.

        Description:
            - Creates a mark on a specified waveform or all waveforms in a column.

        Usage:
            - Using the ``.write(value)`` method will send the ``MARK:CREATE value`` command.

        SCPI Syntax:
            ```
            - MARK:CREATE {CH<x>|MATH|B<x>|REF<x>|COLUMN|DIGItal}
            ```

        Info:
            - ``CH<x>`` creates the mark on a channel waveform, where <x> is the channel number.
            - ``MATH`` creates the mark on the math waveform.
            - ``B<x>`` creates the mark on a bus waveform, where <x> is the bus number.
            - ``REF<x>`` creates the mark on a reference waveform, where <x> is the reference
              waveform number.
            - ``COLUMN`` creates marks on all waveforms in the current zoom pixel column.
            - ``DIGItal`` creates marks on a digital waveform. (An error will result if no digital
              channel is turned on.).
        """
        return self._create

    @property
    def delete(self) -> MarkDelete:
        """Return the ``MARK:DELEte`` command.

        Description:
            - Deletes a mark on a particular waveform, all waveforms in a column, the selected mark,
              or all marks.

        Usage:
            - Using the ``.write(value)`` method will send the ``MARK:DELEte value`` command.

        SCPI Syntax:
            ```
            - MARK:DELEte {CH<x>|MATH|B<x>|REF<x>|COLUMN|DIGItal}
            ```

        Info:
            - ``CH<x>`` deletes the mark on a channel waveform, where <x> is the channel number.
            - ``MATH`` deletes the mark on the math waveform.
            - ``B<x>`` deletes the mark on a bus waveform, where <x> is the bus number.
            - ``REF<x>`` deletes the mark on a reference waveform, where <x> is the reference
              waveform number.
            - ``COLUMN`` deletes marks on all waveforms in the current zoom pixel column.
            - ``DIGItal`` deletes all marks on all digital channels.
        """
        return self._delete

    @property
    def free(self) -> MarkFree:
        """Return the ``MARK:FREE`` command.

        Description:
            - Returns how many marks are available for use. There can be a total of 1,024 marks
              returned.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:FREE?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:FREE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:FREE?
            ```
        """
        return self._free

    @property
    def selected(self) -> MarkSelected:
        """Return the ``MARK:SELected`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELected?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELected?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.end``: The ``MARK:SELected:END`` command.
            - ``.focus``: The ``MARK:SELected:FOCUS`` command.
            - ``.marksincolumn``: The ``MARK:SELected:MARKSINCOLumn`` command.
            - ``.owner``: The ``MARK:SELected:OWNer`` command.
            - ``.source``: The ``MARK:SELected:SOURCE`` command.
            - ``.start``: The ``MARK:SELected:STARt`` command.
            - ``.state``: The ``MARK:SELected:STATe`` command.
            - ``.zoom``: The ``MARK:SELected:ZOOm`` command tree.
        """
        return self._selected

    @property
    def total(self) -> MarkTotal:
        """Return the ``MARK:TOTal`` command.

        Description:
            - Returns how many marks are currently in use. There can be a total of 1,024 marks
              returned.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:TOTal?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:TOTal?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:TOTal?
            ```
        """
        return self._total



"""The select commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SELect:CH<x> {ON|OFF|1|0}
    - SELect:CH<x>?
    - SELect:DCH<x>:DAll {ON|OFF|<NR1>}
    - SELect:DCH<x>:DAll?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedChannel,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SelectDchItemDall(SCPICmdWrite, SCPICmdRead):
    """The ``SELect:DCH<x>:DAll`` command.

    Description:
        - This command turns on or off all constituent digital channels.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:DCH<x>:DAll?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:DCH<x>:DAll?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:DCH<x>:DAll value`` command.

    SCPI Syntax:
        ```
        - SELect:DCH<x>:DAll {ON|OFF|<NR1>}
        - SELect:DCH<x>:DAll?
        ```

    Info:
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.
        - ``<NR1>`` = 0 disables the specified digital channel; any other value turns the on the
          digital channel.
        - ``OFF`` turns off the specified digital channel.
        - ``ON`` turns on the specified digital channel.
    """


class SelectDchItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``SELect:DCH<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:DCH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:DCH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Info:
        - ``DCH<x>`` specifies the digital channel. The supported value is 1.

    Properties:
        - ``.dall``: The ``SELect:DCH<x>:DAll`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._dall = SelectDchItemDall(device, f"{self._cmd_syntax}:DAll")

    @property
    def dall(self) -> SelectDchItemDall:
        """Return the ``SELect:DCH<x>:DAll`` command.

        Description:
            - This command turns on or off all constituent digital channels.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:DCH<x>:DAll?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:DCH<x>:DAll?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:DCH<x>:DAll value`` command.

        SCPI Syntax:
            ```
            - SELect:DCH<x>:DAll {ON|OFF|<NR1>}
            - SELect:DCH<x>:DAll?
            ```

        Info:
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.
            - ``<NR1>`` = 0 disables the specified digital channel; any other value turns the on the
              digital channel.
            - ``OFF`` turns off the specified digital channel.
            - ``ON`` turns on the specified digital channel.
        """
        return self._dall


class SelectChannel(ValidatedChannel, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:CH<x>`` command.

    Description:
        - Turns the display of the channel <x> waveform on or off, where <x > is the channel number.
          This command also resets the acquisition. The query returns whether the channel is on or
          off but does not indicate whether it is the specified waveform.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

    SCPI Syntax:
        ```
        - SELect:CH<x> {ON|OFF|1|0}
        - SELect:CH<x>?
        ```

    Info:
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``1`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``0`` turns off the display of the specified waveform.
    """


class Select(SCPICmdRead):
    """The ``SELect`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SELect?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.ch``: The ``SELect:CH<x>`` command.
        - ``.dch``: The ``SELect:DCH<x>`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SELect") -> None:
        super().__init__(device, cmd_syntax)
        self._ch: Dict[int, SelectChannel] = DefaultDictPassKeyToFactory(
            lambda x: SelectChannel(device, f"{self._cmd_syntax}:CH{x}")
        )
        self._dch: Dict[int, SelectDchItem] = DefaultDictPassKeyToFactory(
            lambda x: SelectDchItem(device, f"{self._cmd_syntax}:DCH{x}")
        )

    @property
    def ch(self) -> Dict[int, SelectChannel]:
        """Return the ``SELect:CH<x>`` command.

        Description:
            - Turns the display of the channel <x> waveform on or off, where <x > is the channel
              number. This command also resets the acquisition. The query returns whether the
              channel is on or off but does not indicate whether it is the specified waveform.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

        SCPI Syntax:
            ```
            - SELect:CH<x> {ON|OFF|1|0}
            - SELect:CH<x>?
            ```

        Info:
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``1`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``0`` turns off the display of the specified waveform.
        """
        return self._ch

    @property
    def dch(self) -> Dict[int, SelectDchItem]:
        """Return the ``SELect:DCH<x>`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:DCH<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:DCH<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Info:
            - ``DCH<x>`` specifies the digital channel. The supported value is 1.

        Sub-properties:
            - ``.dall``: The ``SELect:DCH<x>:DAll`` command.
        """
        return self._dch



"""The status_and_error commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC, AWG5200, AWG5K, AWG5KC, AWG70KA, AWG70KB, AWG7K, AWG7KC, DPO2K, DPO2KB,
DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7AX, DPO7K, DPO7KC, DSA70KC,
DSA70KD, LPD6, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4, MSO4B, MSO4K, MSO4KB,
MSO5, MSO5B, MSO5K, MSO5KB, MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *CLS
    - *ESR?
    - *STB?
    - *WAI
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Wai(SCPICmdWriteNoArguments):
    """The ``*WAI`` command.

    Description:
        - The ``*WAI`` (Wait) command (no query form) prevents the instrument from executing further
          commands or queries until all pending commands that generate an OPC message are complete.
          This command allows you to synchronize the operation of the instrument with your
          application program. For more information, refer to Synchronization Methods.

    Usage:
        - Using the ``.write()`` method will send the ``*WAI`` command.

    SCPI Syntax:
        ```
        - *WAI
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*WAI") -> None:
        super().__init__(device, cmd_syntax)


class Stb(SCPICmdRead):
    """The ``*STB`` command.

    Description:
        - The ``*STB?`` (Read Status Byte) query returns the contents of the Status Byte Register
          (SBR) using the Master Summary Status (MSS) bit. For more information, refer to Registers.

    Usage:
        - Using the ``.query()`` method will send the ``*STB?`` query.
        - Using the ``.verify(value)`` method will send the ``*STB?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - *STB?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*STB") -> None:
        super().__init__(device, cmd_syntax)


class Esr(SCPICmdRead):
    """The ``*ESR`` command.

    Description:
        - This query-only command returns the contents of the Standard Event Status Register (SESR).
          ``*ESR?`` also clears the SESR (since reading the SESR clears it). For a more detailed
          discussion of the use of these registers, see Registers.

    Usage:
        - Using the ``.query()`` method will send the ``*ESR?`` query.
        - Using the ``.verify(value)`` method will send the ``*ESR?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - *ESR?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*ESR") -> None:
        super().__init__(device, cmd_syntax)


class Cls(SCPICmdWriteNoArguments):
    """The ``*CLS`` command.

    Description:
        - This command (no query form) clears the following: Event Queue Standard Event Status
          Register Status Byte Register (except the MAV bit) If the ``*CLS`` command immediately
          follows an <EOI>, the Output Queue and MAV bit (Status Byte Register bit 4) are also
          cleared. MAV indicates that information is in the output queue. The device clear (DCL)
          control message will clear the output queue and thus MAV. ``*CLS`` does not clear the
          output queue or MAV. ``*CLS`` can suppress a Service Request that is to be generated by an
          ``*OPC``. This will happen if a single sequence acquisition operation is still being
          processed when the ``*CLS`` command is executed.

    Usage:
        - Using the ``.write()`` method will send the ``*CLS`` command.

    SCPI Syntax:
        ```
        - *CLS
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*CLS") -> None:
        super().__init__(device, cmd_syntax)



"""Base SMU 6xxx device driver module."""

from __future__ import annotations

from abc import ABC
from typing import Tuple, TYPE_CHECKING

from tm_devices.driver_mixins.device_control import PIControl
from tm_devices.driver_mixins.shared_implementations.common_pi_system_error_check_mixin import (
    CommonPISystemErrorCheckMixin,
)
from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.source_measure_units.source_measure_unit import SourceMeasureUnit
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813

if TYPE_CHECKING:
    from tm_devices.driver_mixins.shared_implementations.ieee488_2_commands import IEEE4882Commands


@family_base_class
class SMU6xxx(CommonPISystemErrorCheckMixin, PIControl, SourceMeasureUnit, ABC):
    """Base SMU6xxx device driver."""

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names."""
        return tuple(f"SOURCE{x + 1}" for x in range(self.total_channels))

    @property
    def ieee_cmds(self) -> IEEE4882Commands:
        """Return an internal class containing methods for the standard IEEE 488.2 command set."""
        return self._ieee_cmds

    @cached_property
    def total_channels(self) -> int:  # pylint: disable=no-self-use
        """Return the total number of channels (all types)."""
        return 1

    ################################################################################################
    # Public Methods
    ################################################################################################

    ################################################################################################
    # Private Methods
    ################################################################################################



"""The peakstable commands module.

These commands are used in the following models:
TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - PEAKSTABle:ADDNew <QString>
    - PEAKSTABle:DELete <QString>
    - PEAKSTABle:LIST?
    - PEAKSTABle:TABle<x>:FRESolution {AUTO|PRECISE}
    - PEAKSTABle:TABle<x>:FRESolution?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class PeakstableTableItemFresolution(SCPICmdWrite, SCPICmdRead):
    """The ``PEAKSTABle:TABle<x>:FRESolution`` command.

    Description:
        - This command sets or queries the Frequency Resolution state for peak markers table.

    Usage:
        - Using the ``.query()`` method will send the ``PEAKSTABle:TABle<x>:FRESolution?`` query.
        - Using the ``.verify(value)`` method will send the ``PEAKSTABle:TABle<x>:FRESolution?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``PEAKSTABle:TABle<x>:FRESolution value``
          command.

    SCPI Syntax:
        ```
        - PEAKSTABle:TABle<x>:FRESolution {AUTO|PRECISE}
        - PEAKSTABle:TABle<x>:FRESolution?
        ```

    Info:
        - ``AUTO`` shows the frequency with the same precision as shown in the Spectrum View
          display.
        - ``PRECISE`` shows the frequency down to single Hz resolution.
    """


class PeakstableTableItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``PEAKSTABle:TABle<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``PEAKSTABle:TABle<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``PEAKSTABle:TABle<x>?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.fresolution``: The ``PEAKSTABle:TABle<x>:FRESolution`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._fresolution = PeakstableTableItemFresolution(
            device, f"{self._cmd_syntax}:FRESolution"
        )

    @property
    def fresolution(self) -> PeakstableTableItemFresolution:
        """Return the ``PEAKSTABle:TABle<x>:FRESolution`` command.

        Description:
            - This command sets or queries the Frequency Resolution state for peak markers table.

        Usage:
            - Using the ``.query()`` method will send the ``PEAKSTABle:TABle<x>:FRESolution?``
              query.
            - Using the ``.verify(value)`` method will send the ``PEAKSTABle:TABle<x>:FRESolution?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``PEAKSTABle:TABle<x>:FRESolution value`` command.

        SCPI Syntax:
            ```
            - PEAKSTABle:TABle<x>:FRESolution {AUTO|PRECISE}
            - PEAKSTABle:TABle<x>:FRESolution?
            ```

        Info:
            - ``AUTO`` shows the frequency with the same precision as shown in the Spectrum View
              display.
            - ``PRECISE`` shows the frequency down to single Hz resolution.
        """
        return self._fresolution


class PeakstableList(SCPICmdRead):
    """The ``PEAKSTABle:LIST`` command.

    Description:
        - This command deletes the specified peak markers table.

    Usage:
        - Using the ``.query()`` method will send the ``PEAKSTABle:LIST?`` query.
        - Using the ``.verify(value)`` method will send the ``PEAKSTABle:LIST?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - PEAKSTABle:LIST?
        ```
    """


class PeakstableDelete(SCPICmdWrite):
    """The ``PEAKSTABle:DELete`` command.

    Description:
        - This command deletes the specified peak markers table.

    Usage:
        - Using the ``.write(value)`` method will send the ``PEAKSTABle:DELete value`` command.

    SCPI Syntax:
        ```
        - PEAKSTABle:DELete <QString>
        ```

    Info:
        - ``<QString>`` is a quoted string that is the name of the peak markers table to delete. The
          argument is of the form 'TABLE<NR1>', where <NR1> ≥ 1.
    """

    _WRAP_ARG_WITH_QUOTES = True


class PeakstableAddnew(SCPICmdWrite):
    """The ``PEAKSTABle:ADDNew`` command.

    Description:
        - This command adds the specified peak markers table.

    Usage:
        - Using the ``.write(value)`` method will send the ``PEAKSTABle:ADDNew value`` command.

    SCPI Syntax:
        ```
        - PEAKSTABle:ADDNew <QString>
        ```

    Info:
        - ``<QString>`` is a quoted string that is the name of the new peak markers table. The
          argument is of the form 'TABLE<NR1>', where <NR1> ≥ 1.
    """

    _WRAP_ARG_WITH_QUOTES = True


class Peakstable(SCPICmdRead):
    """The ``PEAKSTABle`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``PEAKSTABle?`` query.
        - Using the ``.verify(value)`` method will send the ``PEAKSTABle?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.addnew``: The ``PEAKSTABle:ADDNew`` command.
        - ``.delete``: The ``PEAKSTABle:DELete`` command.
        - ``.list``: The ``PEAKSTABle:LIST`` command.
        - ``.table``: The ``PEAKSTABle:TABle<x>`` command tree.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "PEAKSTABle"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._addnew = PeakstableAddnew(device, f"{self._cmd_syntax}:ADDNew")
        self._delete = PeakstableDelete(device, f"{self._cmd_syntax}:DELete")
        self._list = PeakstableList(device, f"{self._cmd_syntax}:LIST")
        self._table: Dict[int, PeakstableTableItem] = DefaultDictPassKeyToFactory(
            lambda x: PeakstableTableItem(device, f"{self._cmd_syntax}:TABle{x}")
        )

    @property
    def addnew(self) -> PeakstableAddnew:
        """Return the ``PEAKSTABle:ADDNew`` command.

        Description:
            - This command adds the specified peak markers table.

        Usage:
            - Using the ``.write(value)`` method will send the ``PEAKSTABle:ADDNew value`` command.

        SCPI Syntax:
            ```
            - PEAKSTABle:ADDNew <QString>
            ```

        Info:
            - ``<QString>`` is a quoted string that is the name of the new peak markers table. The
              argument is of the form 'TABLE<NR1>', where <NR1> ≥ 1.
        """
        return self._addnew

    @property
    def delete(self) -> PeakstableDelete:
        """Return the ``PEAKSTABle:DELete`` command.

        Description:
            - This command deletes the specified peak markers table.

        Usage:
            - Using the ``.write(value)`` method will send the ``PEAKSTABle:DELete value`` command.

        SCPI Syntax:
            ```
            - PEAKSTABle:DELete <QString>
            ```

        Info:
            - ``<QString>`` is a quoted string that is the name of the peak markers table to delete.
              The argument is of the form 'TABLE<NR1>', where <NR1> ≥ 1.
        """
        return self._delete

    @property
    def list(self) -> PeakstableList:
        """Return the ``PEAKSTABle:LIST`` command.

        Description:
            - This command deletes the specified peak markers table.

        Usage:
            - Using the ``.query()`` method will send the ``PEAKSTABle:LIST?`` query.
            - Using the ``.verify(value)`` method will send the ``PEAKSTABle:LIST?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - PEAKSTABle:LIST?
            ```
        """
        return self._list

    @property
    def table(self) -> Dict[int, PeakstableTableItem]:
        """Return the ``PEAKSTABle:TABle<x>`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``PEAKSTABle:TABle<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``PEAKSTABle:TABle<x>?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.fresolution``: The ``PEAKSTABle:TABle<x>:FRESolution`` command.
        """
        return self._table



"""The autoset commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AUTOSet {EXECute|UNDo}
    - AUTOSet:ENAble {ON|OFF|1|0}
    - AUTOSet:ENAble?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AutosetEnable(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet:ENAble`` command.

    Description:
        - This command sets or queries the Autoset enable/disable feature. This is useful for
          classroom purposes where the instructor wants the students to achieve the desired
          instrument settings without the benefit of the Autoset feature.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:ENAble?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSet:ENAble value`` command.

    SCPI Syntax:
        ```
        - AUTOSet:ENAble {ON|OFF|1|0}
        - AUTOSet:ENAble?
        ```

    Info:
        - ``ON`` or 1 enables Autoset.
        - ``OFF`` or 0 disables Autoset.
    """


class Autoset(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet`` command.

    Description:
        - Sets the vertical, horizontal, and trigger controls of the oscilloscope to automatically
          acquire and display the selected waveform.

    Usage:
        - Using the ``.write(value)`` method will send the ``AUTOSet value`` command.

    SCPI Syntax:
        ```
        - AUTOSet {EXECute|UNDo}
        ```

    Info:
        - ``EXECute`` autosets the displayed waveform.
        - ``UNDo`` restores the oscilloscope settings to those present prior to the autoset
          execution.

    Properties:
        - ``.enable``: The ``AUTOSet:ENAble`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "AUTOSet") -> None:
        super().__init__(device, cmd_syntax)
        self._enable = AutosetEnable(device, f"{self._cmd_syntax}:ENAble")

    @property
    def enable(self) -> AutosetEnable:
        """Return the ``AUTOSet:ENAble`` command.

        Description:
            - This command sets or queries the Autoset enable/disable feature. This is useful for
              classroom purposes where the instructor wants the students to achieve the desired
              instrument settings without the benefit of the Autoset feature.

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:ENAble?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUTOSet:ENAble value`` command.

        SCPI Syntax:
            ```
            - AUTOSet:ENAble {ON|OFF|1|0}
            - AUTOSet:ENAble?
            ```

        Info:
            - ``ON`` or 1 enables Autoset.
            - ``OFF`` or 0 disables Autoset.
        """
        return self._enable



"""The lock commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - LOCk {ALL|NONe}
    - LOCk?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Lock(SCPICmdWrite, SCPICmdRead):
    """The ``LOCk`` command.

    Description:
        - This command enables or disables the touch screen and all front panel buttons and knobs.
          There is no front panel equivalent. When the front panel is locked, the front panel
          commands will not work and will not generate error events. You can work around a locked
          front panel, by using the appropriate programmatic interface commands, instead of the
          front-panel commands. For example, to set the trigger level to 50%, you could use
          ``TRIGger:A SETLevel``. To force a trigger, you could use TRIGger FORCe.

    Usage:
        - Using the ``.query()`` method will send the ``LOCk?`` query.
        - Using the ``.verify(value)`` method will send the ``LOCk?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``LOCk value`` command.

    SCPI Syntax:
        ```
        - LOCk {ALL|NONe}
        - LOCk?
        ```

    Info:
        - ``ALL`` disables all front panel controls and the touch screen.
        - ``NONe`` enables all front panel controls and the touch screen. The UNLock ALL command
          only unlocks the front panel controls.
        - ``LOCk NONe`` command has no effect. For more information, see the ANSI/IEEE Std
          488.1-1987 Standard Digital Interface for Programmable Instrumentation, section 2.8.3 on
          RL State Descriptions.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "LOCk") -> None:
        super().__init__(device, cmd_syntax)



"""The alias commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - ALIas {ON|OFF|<NR1>}
    - ALIas:CATalog?
    - ALIas:DEFine <QString><,>{<QString>|<Block>}
    - ALIas:DEFine? <QString>
    - ALIas:DELEte <QString>
    - ALIas:DELEte:ALL
    - ALIas:DELEte:NAMe <QString>
    - ALIas:STATE {ON|OFF|<NR1>}
    - ALIas:STATE?
    - ALIas?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdReadWithArguments, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AliasState(SCPICmdWrite, SCPICmdRead):
    """The ``ALIas:STATE`` command.

    Description:
        - Turns aliases on or off.

    Usage:
        - Using the ``.query()`` method will send the ``ALIas:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``ALIas:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ALIas:STATE value`` command.

    SCPI Syntax:
        ```
        - ALIas:STATE {ON|OFF|<NR1>}
        - ALIas:STATE?
        ```

    Info:
        - ``OFF`` or <NR1> = 0 turns alias expansion off. If a defined alias is sent when
          ``ALIas:STATE`` is OFF, a command error (102) is generated.
        - ``ON`` or <NR1>0 turns alias expansion on. When a defined alias is received, the specified
          command sequence is substituted for the alias and executed.
    """


class AliasDeleteName(SCPICmdWrite):
    """The ``ALIas:DELEte:NAMe`` command.

    Description:
        - Removes a specified alias.

    Usage:
        - Using the ``.write(value)`` method will send the ``ALIas:DELEte:NAMe value`` command.

    SCPI Syntax:
        ```
        - ALIas:DELEte:NAMe <QString>
        ```

    Info:
        - ``<QString>`` is the name of the alias to remove. Using <QString> must be an existing
          alias.
    """

    _WRAP_ARG_WITH_QUOTES = True


class AliasDeleteAll(SCPICmdWriteNoArguments):
    """The ``ALIas:DELEte:ALL`` command.

    Description:
        - This command deletes all existing aliases.

    Usage:
        - Using the ``.write()`` method will send the ``ALIas:DELEte:ALL`` command.

    SCPI Syntax:
        ```
        - ALIas:DELEte:ALL
        ```
    """


class AliasDelete(SCPICmdWrite, SCPICmdRead):
    """The ``ALIas:DELEte`` command.

    Description:
        - This command removes a specified alias and is identical to ``ALIas:DELEte:NAMe``. An error
          message is generated if the named alias does not exist.

    Usage:
        - Using the ``.write(value)`` method will send the ``ALIas:DELEte value`` command.

    SCPI Syntax:
        ```
        - ALIas:DELEte <QString>
        ```

    Info:
        - ``<QString>`` is the name of the alias to be removed. Using <QString> must be a previously
          defined value.

    Properties:
        - ``.all``: The ``ALIas:DELEte:ALL`` command.
        - ``.name``: The ``ALIas:DELEte:NAMe`` command.
    """

    _WRAP_ARG_WITH_QUOTES = True

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._all = AliasDeleteAll(device, f"{self._cmd_syntax}:ALL")
        self._name = AliasDeleteName(device, f"{self._cmd_syntax}:NAMe")

    @property
    def all(self) -> AliasDeleteAll:
        """Return the ``ALIas:DELEte:ALL`` command.

        Description:
            - This command deletes all existing aliases.

        Usage:
            - Using the ``.write()`` method will send the ``ALIas:DELEte:ALL`` command.

        SCPI Syntax:
            ```
            - ALIas:DELEte:ALL
            ```
        """
        return self._all

    @property
    def name(self) -> AliasDeleteName:
        """Return the ``ALIas:DELEte:NAMe`` command.

        Description:
            - Removes a specified alias.

        Usage:
            - Using the ``.write(value)`` method will send the ``ALIas:DELEte:NAMe value`` command.

        SCPI Syntax:
            ```
            - ALIas:DELEte:NAMe <QString>
            ```

        Info:
            - ``<QString>`` is the name of the alias to remove. Using <QString> must be an existing
              alias.
        """
        return self._name


class AliasDefine(SCPICmdWrite, SCPICmdReadWithArguments):
    """The ``ALIas:DEFine`` command.

    Description:
        - Assigns a sequence of program messages to an alias label. These messages are then
          substituted for the alias whenever it is received as a command or query, provided that
          ``ALIas:STATE`` has been turned on. The query form of this command returns the definitions
          of a selected alias.

    Usage:
        - Using the ``.query(argument)`` method will send the ``ALIas:DEFine? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``ALIas:DEFine? argument``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ALIas:DEFine value`` command.

    SCPI Syntax:
        ```
        - ALIas:DEFine <QString><,>{<QString>|<Block>}
        - ALIas:DEFine? <QString>
        ```

    Info:
        - ``<QString>`` is the alias label.
        - ``<QString>`` or <Block> is a complete sequence of program messages.
    """


class AliasCatalog(SCPICmdRead):
    """The ``ALIas:CATalog`` command.

    Description:
        - This query-only command returns a list of the currently defined alias labels, separated by
          commas. If no aliases are defined, the query returns the string ''.

    Usage:
        - Using the ``.query()`` method will send the ``ALIas:CATalog?`` query.
        - Using the ``.verify(value)`` method will send the ``ALIas:CATalog?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ALIas:CATalog?
        ```
    """


class Alias(SCPICmdWrite, SCPICmdRead):
    """The ``ALIas`` command.

    Description:
        - This command sets or queries the state of alias functionality, and it is identical to the
          ``ALIAS:STATE`` command.

    Usage:
        - Using the ``.query()`` method will send the ``ALIas?`` query.
        - Using the ``.verify(value)`` method will send the ``ALIas?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ALIas value`` command.

    SCPI Syntax:
        ```
        - ALIas {ON|OFF|<NR1>}
        - ALIas?
        ```

    Info:
        - ``OFF`` turns Alias expansion off.
        - ``ON`` turns Alias expansion on. When a defined alias is received, the specified command
          sequence is substituted for the alias and executed.
        - ``<NR1>`` = 0 disables Alias mode; any other value enables Alias mode.

    Properties:
        - ``.catalog``: The ``ALIas:CATalog`` command.
        - ``.define``: The ``ALIas:DEFine`` command.
        - ``.delete``: The ``ALIas:DELEte`` command.
        - ``.state``: The ``ALIas:STATE`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "ALIas") -> None:
        super().__init__(device, cmd_syntax)
        self._catalog = AliasCatalog(device, f"{self._cmd_syntax}:CATalog")
        self._define = AliasDefine(device, f"{self._cmd_syntax}:DEFine")
        self._delete = AliasDelete(device, f"{self._cmd_syntax}:DELEte")
        self._state = AliasState(device, f"{self._cmd_syntax}:STATE")

    @property
    def catalog(self) -> AliasCatalog:
        """Return the ``ALIas:CATalog`` command.

        Description:
            - This query-only command returns a list of the currently defined alias labels,
              separated by commas. If no aliases are defined, the query returns the string ''.

        Usage:
            - Using the ``.query()`` method will send the ``ALIas:CATalog?`` query.
            - Using the ``.verify(value)`` method will send the ``ALIas:CATalog?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ALIas:CATalog?
            ```
        """
        return self._catalog

    @property
    def define(self) -> AliasDefine:
        """Return the ``ALIas:DEFine`` command.

        Description:
            - Assigns a sequence of program messages to an alias label. These messages are then
              substituted for the alias whenever it is received as a command or query, provided that
              ``ALIas:STATE`` has been turned on. The query form of this command returns the
              definitions of a selected alias.

        Usage:
            - Using the ``.query(argument)`` method will send the ``ALIas:DEFine? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the ``ALIas:DEFine? argument``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ALIas:DEFine value`` command.

        SCPI Syntax:
            ```
            - ALIas:DEFine <QString><,>{<QString>|<Block>}
            - ALIas:DEFine? <QString>
            ```

        Info:
            - ``<QString>`` is the alias label.
            - ``<QString>`` or <Block> is a complete sequence of program messages.
        """
        return self._define

    @property
    def delete(self) -> AliasDelete:
        """Return the ``ALIas:DELEte`` command.

        Description:
            - This command removes a specified alias and is identical to ``ALIas:DELEte:NAMe``. An
              error message is generated if the named alias does not exist.

        Usage:
            - Using the ``.write(value)`` method will send the ``ALIas:DELEte value`` command.

        SCPI Syntax:
            ```
            - ALIas:DELEte <QString>
            ```

        Info:
            - ``<QString>`` is the name of the alias to be removed. Using <QString> must be a
              previously defined value.

        Sub-properties:
            - ``.all``: The ``ALIas:DELEte:ALL`` command.
            - ``.name``: The ``ALIas:DELEte:NAMe`` command.
        """
        return self._delete

    @property
    def state(self) -> AliasState:
        """Return the ``ALIas:STATE`` command.

        Description:
            - Turns aliases on or off.

        Usage:
            - Using the ``.query()`` method will send the ``ALIas:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``ALIas:STATE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ALIas:STATE value`` command.

        SCPI Syntax:
            ```
            - ALIas:STATE {ON|OFF|<NR1>}
            - ALIas:STATE?
            ```

        Info:
            - ``OFF`` or <NR1> = 0 turns alias expansion off. If a defined alias is sent when
              ``ALIas:STATE`` is OFF, a command error (102) is generated.
            - ``ON`` or <NR1>0 turns alias expansion on. When a defined alias is received, the
              specified command sequence is substituted for the alias and executed.
        """
        return self._state



"""The miscellaneous commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC,
DSA70KC, DSA70KD, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4K, MSO4KB, MSO5K,
MSO5KB, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *LRN?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Lrn(SCPICmdRead):
    """The ``*LRN`` command.

    Description:
        - This query-only command returns the commands that list the instrument settings, allowing
          you to record or 'learn' the current instrument settings. You can use these commands to
          return the instrument to the state it was in when you made the ``*LRN?`` query. This
          command is identical to the SET command.

    Usage:
        - Using the ``.query()`` method will send the ``*LRN?`` query.
        - Using the ``.verify(value)`` method will send the ``*LRN?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - *LRN?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*LRN") -> None:
        super().__init__(device, cmd_syntax)



# pyright: reportConstantRedefinition=none
"""The localnode commands module.

These commands are used in the following models:
MP5103

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - localnode.license
    - localnode.linefreq
    - localnode.manufacturer
    - localnode.model
    - localnode.prompts
    - localnode.prompts4882
    - localnode.serialno
    - localnode.showerrors
    - localnode.version
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Localnode(BaseTSPCmd):
    """The ``localnode`` command tree.

    Properties and methods:
        - ``.license``: The ``localnode.license`` attribute.
        - ``.linefreq``: The ``localnode.linefreq`` attribute.
        - ``.manufacturer``: The ``localnode.manufacturer`` attribute.
        - ``.model``: The ``localnode.model`` attribute.
        - ``.prompts``: The ``localnode.prompts`` attribute.
        - ``.prompts4882``: The ``localnode.prompts4882`` attribute.
        - ``.serialno``: The ``localnode.serialno`` attribute.
        - ``.showerrors``: The ``localnode.showerrors`` attribute.
        - ``.version``: The ``localnode.version`` attribute.
    """

    def __init__(
        self, device: Optional["TSPControl"] = None, cmd_syntax: str = "localnode"
    ) -> None:
        super().__init__(device, cmd_syntax)

    @property
    def license(self) -> str:
        """Access the ``localnode.license`` attribute.

        Description:
            - This attribute stores the license of the instrument.

        Usage:
            - Accessing this property will send the ``print(localnode.license)`` query.

        TSP Syntax:
            ```
            - print(localnode.license)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".license"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.license)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.license`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def linefreq(self) -> str:
        """Access the ``localnode.linefreq`` attribute.

        Description:
            - This attribute contains the power line frequency detected at power-on.

        Usage:
            - Accessing this property will send the ``print(localnode.linefreq)`` query.

        TSP Syntax:
            ```
            - print(localnode.linefreq)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".linefreq"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.linefreq)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.linefreq`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def manufacturer(self) -> str:
        """Access the ``localnode.manufacturer`` attribute.

        Description:
            - This attribute stores the manufacturer of the instrument.

        Usage:
            - Accessing this property will send the ``print(localnode.manufacturer)`` query.

        TSP Syntax:
            ```
            - print(localnode.manufacturer)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".manufacturer"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.manufacturer)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.manufacturer`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def model(self) -> str:
        """Access the ``localnode.model`` attribute.

        Description:
            - This attribute stores the model number of the instrument.

        Usage:
            - Accessing this property will send the ``print(localnode.model)`` query.

        TSP Syntax:
            ```
            - print(localnode.model)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".model"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.model)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.model`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def prompts(self) -> str:
        """Access the ``localnode.prompts`` attribute.

        Description:
            - This attribute determines if the instrument generates prompts in response to command
              messages.

        Usage:
            - Accessing this property will send the ``print(localnode.prompts)`` query.
            - Setting this property to a value will send the ``localnode.prompts = value`` command.

        TSP Syntax:
            ```
            - localnode.prompts = value
            - print(localnode.prompts)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".prompts"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.prompts)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.prompts`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @prompts.setter
    def prompts(self, value: Union[str, float]) -> None:
        """Access the ``localnode.prompts`` attribute.

        Description:
            - This attribute determines if the instrument generates prompts in response to command
              messages.

        Usage:
            - Accessing this property will send the ``print(localnode.prompts)`` query.
            - Setting this property to a value will send the ``localnode.prompts = value`` command.

        TSP Syntax:
            ```
            - localnode.prompts = value
            - print(localnode.prompts)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".prompts", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.prompts = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.prompts`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def prompts4882(self) -> str:
        """Access the ``localnode.prompts4882`` attribute.

        Description:
            - This attribute enables or disables the generation of prompts for IEEE Std 488.2 common
              commands.

        Usage:
            - Accessing this property will send the ``print(localnode.prompts4882)`` query.
            - Setting this property to a value will send the ``localnode.prompts4882 = value``
              command.

        TSP Syntax:
            ```
            - localnode.prompts4882 = value
            - print(localnode.prompts4882)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".prompts4882"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.prompts4882)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.prompts4882`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @prompts4882.setter
    def prompts4882(self, value: Union[str, float]) -> None:
        """Access the ``localnode.prompts4882`` attribute.

        Description:
            - This attribute enables or disables the generation of prompts for IEEE Std 488.2 common
              commands.

        Usage:
            - Accessing this property will send the ``print(localnode.prompts4882)`` query.
            - Setting this property to a value will send the ``localnode.prompts4882 = value``
              command.

        TSP Syntax:
            ```
            - localnode.prompts4882 = value
            - print(localnode.prompts4882)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".prompts4882", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.prompts4882 = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.prompts4882`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def serialno(self) -> str:
        """Access the ``localnode.serialno`` attribute.

        Description:
            - This attribute stores the serial number of the instrument.

        Usage:
            - Accessing this property will send the ``print(localnode.serialno)`` query.

        TSP Syntax:
            ```
            - print(localnode.serialno)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".serialno"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.serialno)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.serialno`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def showerrors(self) -> str:
        """Access the ``localnode.showerrors`` attribute.

        Description:
            - This attribute sets whether or not the instrument automatically sends generated
              errors.

        Usage:
            - Accessing this property will send the ``print(localnode.showerrors)`` query.
            - Setting this property to a value will send the ``localnode.showerrors = value``
              command.

        TSP Syntax:
            ```
            - localnode.showerrors = value
            - print(localnode.showerrors)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".showerrors"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.showerrors)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.showerrors`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @showerrors.setter
    def showerrors(self, value: Union[str, float]) -> None:
        """Access the ``localnode.showerrors`` attribute.

        Description:
            - This attribute sets whether or not the instrument automatically sends generated
              errors.

        Usage:
            - Accessing this property will send the ``print(localnode.showerrors)`` query.
            - Setting this property to a value will send the ``localnode.showerrors = value``
              command.

        TSP Syntax:
            ```
            - localnode.showerrors = value
            - print(localnode.showerrors)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".showerrors", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.showerrors = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.showerrors`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def version(self) -> str:
        """Access the ``localnode.version`` attribute.

        Description:
            - This attribute stores the firmware revision level.

        Usage:
            - Accessing this property will send the ``print(localnode.version)`` query.

        TSP Syntax:
            ```
            - print(localnode.version)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".version"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.version)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.version`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The curve commands module.

These commands are used in the following models:
LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CURVe?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Curve(SCPICmdRead):
    """The ``CURVe`` command.

    Description:
        - This command transfers waveform data from the instrument. Each waveform that is
          transferred has an associated waveform preamble that contains information such as data
          format and scale. The ``CURVe?`` query transfers data from the instrument. The data source
          is specified by the ``DATa:SOUrce`` command. The first and last data points are specified
          by the ``DATa:STARt`` and ``DATa:STOP`` commands. For digital sources, ``CH<x>_D<x>`` or
          CH<x> _DALL, when the ``:DATa:WIDth`` is 1, the returned data is state only. When the
          ``:DATa:WIDth`` is 2, the returned data is transition data with 2 bits per digital channel
          representing the transition information as follows: 0 0 low 0 1 high 1 1 multiple
          transitions in interval ending with high 1 0 multiple transitions in interval ending with
          low For individual digital channels (such as ``CH<x>_D<x>`` ), ``:DATa:WIDth 2`` provides
          the 2-bit transition data with the upper 14 bits zero. ``:DATa:WIDth 1`` provides only the
          state in the LSB with the upper 7 bits all zero. For ``CH<x>_DAll`` sources,
          ``:DATa:WIDth 2`` provides the 2-bit transition data for each of the 8 constituent
          channels with the D7 bit represented in the 2 most significant bits, D6 in the next 2, and
          so on. ``:DATa:WIDth 1`` provides the states of each of the 8 constituent channels with D7
          represented in the most significant bit and D0 in the least significant bit. Depending on
          the sample rate, multi-transition data may not be available and ``:CURVe?`` queries for
          digital channels with ``:DATa:WIDth 2`` may result in a warning event 'Execution warning:
          Multi-transition data not available'. In this case, the transition data returned will be 0
          0 or 0 1. For MATH sources, only 8-byte double precision floating point data is returned
          in ``:CURVe?`` queries. A Fast Acquisition waveform Pixmap data is a 500 (vertical) by
          1000 (horizontal) point bitmap. Each point represents display intensity for that screen
          location. 500 (vertical) which is the row count in the bitmap, might vary based on how
          many channels enabled from same FastAcq group. To query and get the Fast Acq Pixel Map
          data, the following command set should be sent: ``ACQuire:FASTAcq:STATE ON``
          ``DATA:MODe PIXmap`` When the FastAcq is on, Curve? on Ch<x> will return pixmap data (if
          ``DATA:MODe`` is PIXmap). The number of rows in the pixmap will vary based on how many
          ch<x> sources are turned on and how they are grouped in acquisition HW. The grouping can
          vary from model to model. The number of columns in pixmap data is fixed to 1000. For
          example, on a MSO58 instrument, Ch1 to Ch4 is 'group1' and Ch5 to Ch8 is 'group2'. If Ch1
          is turned on (in group1) then Ch1 rows will be 500. If Ch2 and Ch3 are turned on (in
          group1) then Ch2 and Ch3 rows will be 250 each. When all Ch1/2/3/4 are turned on (in
          group1) then 125 rows per channel. If Ch1 (in group1) and Ch8 (in group2) are turned on
          then 500 rows will be returned for each channel. To calculate the number of rows, you can
          use- (number of bytes from curve header/``BYT_NR``)/1000. Note: Curve data is transferred
          from the instrument asynchronously and, depending upon the length of the curve record,
          such transfers can require several seconds to complete. During this time, the instrument
          will not respond to usercontrols. You can interrupt these asynchronous data transfers by
          sending adevice clear message to the instrument or by interrupting the query with another
          command or query. Verify that curve data is completely transferred.

    Usage:
        - Using the ``.query()`` method will send the ``CURVe?`` query.
        - Using the ``.verify(value)`` method will send the ``CURVe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CURVe?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "CURVe") -> None:
        super().__init__(device, cmd_syntax)



"""The calibration commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC, AWG5K, AWG5KC, AWG7K, AWG7KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CALibration:ALL
    - CALibration:ALL?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class CalibrationAll(SCPICmdWriteNoArguments, SCPICmdRead):
    """The ``CALibration:ALL`` command.

    Description:
        - The CALibration[``:ALL``] command performs an internal calibration. The
          CALibration[``:ALL``]? command performs an internal calibration and returns 0 (Pass) or a
          calibration error code.

    Usage:
        - Using the ``.query()`` method will send the ``CALibration:ALL?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibration:ALL?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write()`` method will send the ``CALibration:ALL`` command.

    SCPI Syntax:
        ```
        - CALibration:ALL
        - CALibration:ALL?
        ```
    """


class Calibration(SCPICmdRead):
    """The ``CALibration`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CALibration?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibration?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.all``: The ``CALibration:ALL`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "CALibration"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._all = CalibrationAll(device, f"{self._cmd_syntax}:ALL")

    @property
    def all(self) -> CalibrationAll:
        """Return the ``CALibration:ALL`` command.

        Description:
            - The CALibration[``:ALL``] command performs an internal calibration. The
              CALibration[``:ALL``]? command performs an internal calibration and returns 0 (Pass)
              or a calibration error code.

        Usage:
            - Using the ``.query()`` method will send the ``CALibration:ALL?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibration:ALL?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write()`` method will send the ``CALibration:ALL`` command.

        SCPI Syntax:
            ```
            - CALibration:ALL
            - CALibration:ALL?
            ```
        """
        return self._all



# pyright: reportConstantRedefinition=none
"""The fan commands module.

These commands are used in the following models:
DMM7510

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - fan.level
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Fan(BaseTSPCmd):
    """The ``fan`` command tree.

    Properties and methods:
        - ``.level``: The ``fan.level`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "fan") -> None:
        super().__init__(device, cmd_syntax)

    @property
    def level(self) -> str:
        """Access the ``fan.level`` attribute.

        Description:
            - This attribute sets the speed of the instrument fan.

        Usage:
            - Accessing this property will send the ``print(fan.level)`` query.
            - Setting this property to a value will send the ``fan.level = value`` command.

        TSP Syntax:
            ```
            - fan.level = value
            - print(fan.level)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".level"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.level)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.level`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @level.setter
    def level(self, value: Union[str, float]) -> None:
        """Access the ``fan.level`` attribute.

        Description:
            - This attribute sets the speed of the instrument fan.

        Usage:
            - Accessing this property will send the ``print(fan.level)`` query.
            - Setting this property to a value will send the ``fan.level = value`` command.

        TSP Syntax:
            ```
            - fan.level = value
            - print(fan.level)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".level", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.level = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.level`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The rem commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - REM <QString>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Rem(SCPICmdWrite):
    """The ``REM`` command.

    Description:
        - This command (no query form) embeds a comment within programs as a means of internally
          documenting the programs. This is how to embed comments in a.set file. The instrument
          ignores these embedded comment lines.

    Usage:
        - Using the ``.write(value)`` method will send the ``REM value`` command.

    SCPI Syntax:
        ```
        - REM <QString>
        ```

    Info:
        - ``<QString>`` is a string that can contain a maximum of 80 characters.
    """

    _WRAP_ARG_WITH_QUOTES = True

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "REM") -> None:
        super().__init__(device, cmd_syntax)



"""The status_and_error commands module.

These commands are used in the following models:
AWG5200, AWG5K, AWG5KC, AWG70KA, AWG70KB, AWG7K, AWG7KC, DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K,
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MDO3, MDO3K, MDO4K,
MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4K, MSO4KB, MSO5K, MSO5KB, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *ESE <NR1>
    - *ESE?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Ese(SCPICmdWrite, SCPICmdRead):
    """The ``*ESE`` command.

    Description:
        - This command sets and queries the bits in the Event Status Enable Register (ESER). The
          ESER prevents events from being reported to the Status Byte Register (STB). For a more
          detailed discussion of the use of these registers, see Registers.

    Usage:
        - Using the ``.query()`` method will send the ``*ESE?`` query.
        - Using the ``.verify(value)`` method will send the ``*ESE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``*ESE value`` command.

    SCPI Syntax:
        ```
        - *ESE <NR1>
        - *ESE?
        ```

    Info:
        - ``<NR1>`` specifies the binary bits of the ESER according to this value, which ranges from
          0 through 255.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*ESE") -> None:
        super().__init__(device, cmd_syntax)



"""The application commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - APPLication:ACTivate <string>
    - APPLication:SCOPEAPP:WINDOW {FULLSCREEN|MINIMIZED}
    - APPLication:SCOPEAPP:WINDOW?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class ApplicationScopeappWindow(SCPICmdWrite, SCPICmdRead):
    """The ``APPLication:SCOPEAPP:WINDOW`` command.

    Description:
        - Sets or queries whether the oscilloscope application is minimized or displayed full
          screen.

    Usage:
        - Using the ``.query()`` method will send the ``APPLication:SCOPEAPP:WINDOW?`` query.
        - Using the ``.verify(value)`` method will send the ``APPLication:SCOPEAPP:WINDOW?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``APPLication:SCOPEAPP:WINDOW value``
          command.

    SCPI Syntax:
        ```
        - APPLication:SCOPEAPP:WINDOW {FULLSCREEN|MINIMIZED}
        - APPLication:SCOPEAPP:WINDOW?
        ```

    Info:
        - ``FULLSCREEN`` sets the oscilloscope display to fill the oscilloscope screen.
        - ``MINIMIZED`` minimizes the oscilloscope display to an icon at the bottom of the screen.
    """


class ApplicationScopeapp(SCPICmdRead):
    """The ``APPLication:SCOPEAPP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``APPLication:SCOPEAPP?`` query.
        - Using the ``.verify(value)`` method will send the ``APPLication:SCOPEAPP?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.window``: The ``APPLication:SCOPEAPP:WINDOW`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._window = ApplicationScopeappWindow(device, f"{self._cmd_syntax}:WINDOW")

    @property
    def window(self) -> ApplicationScopeappWindow:
        """Return the ``APPLication:SCOPEAPP:WINDOW`` command.

        Description:
            - Sets or queries whether the oscilloscope application is minimized or displayed full
              screen.

        Usage:
            - Using the ``.query()`` method will send the ``APPLication:SCOPEAPP:WINDOW?`` query.
            - Using the ``.verify(value)`` method will send the ``APPLication:SCOPEAPP:WINDOW?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``APPLication:SCOPEAPP:WINDOW value``
              command.

        SCPI Syntax:
            ```
            - APPLication:SCOPEAPP:WINDOW {FULLSCREEN|MINIMIZED}
            - APPLication:SCOPEAPP:WINDOW?
            ```

        Info:
            - ``FULLSCREEN`` sets the oscilloscope display to fill the oscilloscope screen.
            - ``MINIMIZED`` minimizes the oscilloscope display to an icon at the bottom of the
              screen.
        """
        return self._window


class ApplicationActivate(SCPICmdWrite):
    """The ``APPLication:ACTivate`` command.

    Description:
        - For legacy Java based applications, starts the application specified in the string. The
          available applications depend on the oscilloscope model and installed options. (DPOJET and
          DPOJET-based applications do not require this command and are started automatically by
          sending them any command, such as DPOJET: VERsion? )

    Usage:
        - Using the ``.write(value)`` method will send the ``APPLication:ACTivate value`` command.

    SCPI Syntax:
        ```
        - APPLication:ACTivate <string>
        ```

    Info:
        - ``<string>`` is the name of the application that you want to start. Enter the application
          name exactly as it appears in the oscilloscope Analyze menu.
    """


class Application(SCPICmdRead):
    """The ``APPLication`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``APPLication?`` query.
        - Using the ``.verify(value)`` method will send the ``APPLication?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.activate``: The ``APPLication:ACTivate`` command.
        - ``.scopeapp``: The ``APPLication:SCOPEAPP`` command tree.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "APPLication"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._activate = ApplicationActivate(device, f"{self._cmd_syntax}:ACTivate")
        self._scopeapp = ApplicationScopeapp(device, f"{self._cmd_syntax}:SCOPEAPP")

    @property
    def activate(self) -> ApplicationActivate:
        """Return the ``APPLication:ACTivate`` command.

        Description:
            - For legacy Java based applications, starts the application specified in the string.
              The available applications depend on the oscilloscope model and installed options.
              (DPOJET and DPOJET-based applications do not require this command and are started
              automatically by sending them any command, such as DPOJET: VERsion? )

        Usage:
            - Using the ``.write(value)`` method will send the ``APPLication:ACTivate value``
              command.

        SCPI Syntax:
            ```
            - APPLication:ACTivate <string>
            ```

        Info:
            - ``<string>`` is the name of the application that you want to start. Enter the
              application name exactly as it appears in the oscilloscope Analyze menu.
        """
        return self._activate

    @property
    def scopeapp(self) -> ApplicationScopeapp:
        """Return the ``APPLication:SCOPEAPP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``APPLication:SCOPEAPP?`` query.
            - Using the ``.verify(value)`` method will send the ``APPLication:SCOPEAPP?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.window``: The ``APPLication:SCOPEAPP:WINDOW`` command.
        """
        return self._scopeapp



"""The slist commands module.

These commands are used in the following models:
AWG5K, AWG5KC, AWG7K, AWG7KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SLISt:NAME? <Index>
    - SLISt:SIZE?
    - SLISt:SUBSequence:DELete {<subseq_name>|ALL}
    - SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt <subseq_name>,<NR1>
    - SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt? <subseq_name>
    - SLISt:SUBSequence:ELEMent[n]:WAVeform[n] <subseq_name>,<wfm_name>
    - SLISt:SUBSequence:ELEMent[n]:WAVeform[n]? <subseq_name>
    - SLISt:SUBSequence:LENGth <subseq_name>,<NR1>
    - SLISt:SUBSequence:LENGth? <subseq_name>
    - SLISt:SUBSequence:NEW <subseq_name>,<length>
    - SLISt:SUBSequence:TSTamp? <subseq_name>
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdReadWithArguments,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SlistSubsequenceTstamp(SCPICmdReadWithArguments):
    """The ``SLISt:SUBSequence:TSTamp`` command.

    Description:
        - This query returns the time stamp of the subsequence. Time stamp is updated whenever the
          subsequence is created or changed. It is not updated when it is renamed. It returns date
          as a string in the form 'yyyy/mm/dd ``hh:mm:ss``' (a white space between data and time).

    Usage:
        - Using the ``.query(argument)`` method will send the ``SLISt:SUBSequence:TSTamp? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SUBSequence:TSTamp? argument`` query and raise an AssertionError if the returned
          value does not match ``value``.

    SCPI Syntax:
        ```
        - SLISt:SUBSequence:TSTamp? <subseq_name>
        ```

    Info:
        - ``<subseq_name>`` ::=<string>.
    """


class SlistSubsequenceNew(SCPICmdWrite):
    """The ``SLISt:SUBSequence:NEW`` command.

    Description:
        - This command creates a new subsequence.

    Usage:
        - Using the ``.write(value)`` method will send the ``SLISt:SUBSequence:NEW value`` command.

    SCPI Syntax:
        ```
        - SLISt:SUBSequence:NEW <subseq_name>,<length>
        ```

    Info:
        - ``<subseq_name>`` ::=<string>.
    """


class SlistSubsequenceLength(SCPICmdWrite, SCPICmdReadWithArguments):
    """The ``SLISt:SUBSequence:LENGth`` command.

    Description:
        - This command and query sets or returns the size of the subsequence.

    Usage:
        - Using the ``.query(argument)`` method will send the ``SLISt:SUBSequence:LENGth? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SUBSequence:LENGth? argument`` query and raise an AssertionError if the returned
          value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SLISt:SUBSequence:LENGth value``
          command.

    SCPI Syntax:
        ```
        - SLISt:SUBSequence:LENGth <subseq_name>,<NR1>
        - SLISt:SUBSequence:LENGth? <subseq_name>
        ```

    Info:
        - ``<subseq_name>`` ::=<string>.
    """


class SlistSubsequenceElementItemWaveformItem(
    ValidatedDynamicNumberCmd, SCPICmdWrite, SCPICmdReadWithArguments
):
    """The ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n]`` command.

    Description:
        - This command and query sets or returns the waveform for an element of the subsequence. The
          value of n = 1 | 2 | 3 | 4 based on the model. If suffix is not specified, the value of n
          is 1. The value of 'n' indicates which channel will output the waveform when the sequence
          is run.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n]? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n]? argument`` query and raise an AssertionError
          if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n] value`` command.

    SCPI Syntax:
        ```
        - SLISt:SUBSequence:ELEMent[n]:WAVeform[n] <subseq_name>,<wfm_name>
        - SLISt:SUBSequence:ELEMent[n]:WAVeform[n]? <subseq_name>
        ```

    Info:
        - ``<subseq_name>`` ::=<string>.
        - ``<wfm_name>`` ::=<string>.
    """


class SlistSubsequenceElementItemLoopCount(SCPICmdWrite, SCPICmdReadWithArguments):
    """The ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt`` command.

    Description:
        - This command and query sets or returns the loop count for the specified subsequence
          element. The loop count is an integer.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt? argument`` query and raise an AssertionError if
          the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt value`` command.

    SCPI Syntax:
        ```
        - SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt <subseq_name>,<NR1>
        - SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt? <subseq_name>
        ```
    """


class SlistSubsequenceElementItemLoop(SCPICmdRead):
    """The ``SLISt:SUBSequence:ELEMent[n]:LOOP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SLISt:SUBSequence:ELEMent[n]:LOOP?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SUBSequence:ELEMent[n]:LOOP?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.count``: The ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._count = SlistSubsequenceElementItemLoopCount(device, f"{self._cmd_syntax}:COUNt")

    @property
    def count(self) -> SlistSubsequenceElementItemLoopCount:
        """Return the ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt`` command.

        Description:
            - This command and query sets or returns the loop count for the specified subsequence
              element. The loop count is an integer.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt value`` command.

        SCPI Syntax:
            ```
            - SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt <subseq_name>,<NR1>
            - SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt? <subseq_name>
            ```
        """
        return self._count


class SlistSubsequenceElementItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``SLISt:SUBSequence:ELEMent[n]`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SLISt:SUBSequence:ELEMent[n]?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SUBSequence:ELEMent[n]?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.loop``: The ``SLISt:SUBSequence:ELEMent[n]:LOOP`` command tree.
        - ``.waveform``: The ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n]`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._loop = SlistSubsequenceElementItemLoop(device, f"{self._cmd_syntax}:LOOP")
        self._waveform: Dict[int, SlistSubsequenceElementItemWaveformItem] = (
            DefaultDictPassKeyToFactory(
                lambda x: SlistSubsequenceElementItemWaveformItem(
                    device, f"{self._cmd_syntax}:WAVeform{x}"
                )
            )
        )

    @property
    def loop(self) -> SlistSubsequenceElementItemLoop:
        """Return the ``SLISt:SUBSequence:ELEMent[n]:LOOP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SLISt:SUBSequence:ELEMent[n]:LOOP?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SLISt:SUBSequence:ELEMent[n]:LOOP?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.count``: The ``SLISt:SUBSequence:ELEMent[n]:LOOP:COUNt`` command.
        """
        return self._loop

    @property
    def waveform(self) -> Dict[int, SlistSubsequenceElementItemWaveformItem]:
        """Return the ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n]`` command.

        Description:
            - This command and query sets or returns the waveform for an element of the subsequence.
              The value of n = 1 | 2 | 3 | 4 based on the model. If suffix is not specified, the
              value of n is 1. The value of 'n' indicates which channel will output the waveform
              when the sequence is run.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n]? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n]? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n] value`` command.

        SCPI Syntax:
            ```
            - SLISt:SUBSequence:ELEMent[n]:WAVeform[n] <subseq_name>,<wfm_name>
            - SLISt:SUBSequence:ELEMent[n]:WAVeform[n]? <subseq_name>
            ```

        Info:
            - ``<subseq_name>`` ::=<string>.
            - ``<wfm_name>`` ::=<string>.
        """
        return self._waveform


class SlistSubsequenceDelete(SCPICmdWrite):
    """The ``SLISt:SUBSequence:DELete`` command.

    Description:
        - This command deletes the subsequence from the currently loaded setup.

    Usage:
        - Using the ``.write(value)`` method will send the ``SLISt:SUBSequence:DELete value``
          command.

    SCPI Syntax:
        ```
        - SLISt:SUBSequence:DELete {<subseq_name>|ALL}
        ```

    Info:
        - ``<subseq_name>`` ::=<string>.
    """


class SlistSubsequence(SCPICmdRead):
    """The ``SLISt:SUBSequence`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SLISt:SUBSequence?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SUBSequence?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.delete``: The ``SLISt:SUBSequence:DELete`` command.
        - ``.element``: The ``SLISt:SUBSequence:ELEMent[n]`` command tree.
        - ``.length``: The ``SLISt:SUBSequence:LENGth`` command.
        - ``.new``: The ``SLISt:SUBSequence:NEW`` command.
        - ``.tstamp``: The ``SLISt:SUBSequence:TSTamp`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._delete = SlistSubsequenceDelete(device, f"{self._cmd_syntax}:DELete")
        self._element: Dict[int, SlistSubsequenceElementItem] = DefaultDictPassKeyToFactory(
            lambda x: SlistSubsequenceElementItem(device, f"{self._cmd_syntax}:ELEMent{x}")
        )
        self._length = SlistSubsequenceLength(device, f"{self._cmd_syntax}:LENGth")
        self._new = SlistSubsequenceNew(device, f"{self._cmd_syntax}:NEW")
        self._tstamp = SlistSubsequenceTstamp(device, f"{self._cmd_syntax}:TSTamp")

    @property
    def delete(self) -> SlistSubsequenceDelete:
        """Return the ``SLISt:SUBSequence:DELete`` command.

        Description:
            - This command deletes the subsequence from the currently loaded setup.

        Usage:
            - Using the ``.write(value)`` method will send the ``SLISt:SUBSequence:DELete value``
              command.

        SCPI Syntax:
            ```
            - SLISt:SUBSequence:DELete {<subseq_name>|ALL}
            ```

        Info:
            - ``<subseq_name>`` ::=<string>.
        """
        return self._delete

    @property
    def element(self) -> Dict[int, SlistSubsequenceElementItem]:
        """Return the ``SLISt:SUBSequence:ELEMent[n]`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SLISt:SUBSequence:ELEMent[n]?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SUBSequence:ELEMent[n]?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.loop``: The ``SLISt:SUBSequence:ELEMent[n]:LOOP`` command tree.
            - ``.waveform``: The ``SLISt:SUBSequence:ELEMent[n]:WAVeform[n]`` command.
        """
        return self._element

    @property
    def length(self) -> SlistSubsequenceLength:
        """Return the ``SLISt:SUBSequence:LENGth`` command.

        Description:
            - This command and query sets or returns the size of the subsequence.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SUBSequence:LENGth? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SUBSequence:LENGth? argument`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SLISt:SUBSequence:LENGth value``
              command.

        SCPI Syntax:
            ```
            - SLISt:SUBSequence:LENGth <subseq_name>,<NR1>
            - SLISt:SUBSequence:LENGth? <subseq_name>
            ```

        Info:
            - ``<subseq_name>`` ::=<string>.
        """
        return self._length

    @property
    def new(self) -> SlistSubsequenceNew:
        """Return the ``SLISt:SUBSequence:NEW`` command.

        Description:
            - This command creates a new subsequence.

        Usage:
            - Using the ``.write(value)`` method will send the ``SLISt:SUBSequence:NEW value``
              command.

        SCPI Syntax:
            ```
            - SLISt:SUBSequence:NEW <subseq_name>,<length>
            ```

        Info:
            - ``<subseq_name>`` ::=<string>.
        """
        return self._new

    @property
    def tstamp(self) -> SlistSubsequenceTstamp:
        """Return the ``SLISt:SUBSequence:TSTamp`` command.

        Description:
            - This query returns the time stamp of the subsequence. Time stamp is updated whenever
              the subsequence is created or changed. It is not updated when it is renamed. It
              returns date as a string in the form 'yyyy/mm/dd ``hh:mm:ss``' (a white space between
              data and time).

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SUBSequence:TSTamp? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SUBSequence:TSTamp? argument`` query and raise an AssertionError if the
              returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SLISt:SUBSequence:TSTamp? <subseq_name>
            ```

        Info:
            - ``<subseq_name>`` ::=<string>.
        """
        return self._tstamp


class SlistSize(SCPICmdRead):
    """The ``SLISt:SIZE`` command.

    Description:
        - This command returns the number sequences in sequence list.

    Usage:
        - Using the ``.query()`` method will send the ``SLISt:SIZE?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SIZE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SLISt:SIZE?
        ```
    """


class SlistName(SCPICmdReadWithArguments):
    """The ``SLISt:NAME`` command.

    Description:
        - The query returns the name of the subsequence corresponding to the specified index in the
          subsequence list.

    Usage:
        - Using the ``.query(argument)`` method will send the ``SLISt:NAME? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``SLISt:NAME? argument`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SLISt:NAME? <Index>
        ```

    Info:
        - ``<Index>`` ::=<NR1>.
    """


class Slist(SCPICmdRead):
    """The ``SLISt`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SLISt?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.name``: The ``SLISt:NAME`` command.
        - ``.size``: The ``SLISt:SIZE`` command.
        - ``.subsequence``: The ``SLISt:SUBSequence`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SLISt") -> None:
        super().__init__(device, cmd_syntax)
        self._name = SlistName(device, f"{self._cmd_syntax}:NAME")
        self._size = SlistSize(device, f"{self._cmd_syntax}:SIZE")
        self._subsequence = SlistSubsequence(device, f"{self._cmd_syntax}:SUBSequence")

    @property
    def name(self) -> SlistName:
        """Return the ``SLISt:NAME`` command.

        Description:
            - The query returns the name of the subsequence corresponding to the specified index in
              the subsequence list.

        Usage:
            - Using the ``.query(argument)`` method will send the ``SLISt:NAME? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the ``SLISt:NAME? argument``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SLISt:NAME? <Index>
            ```

        Info:
            - ``<Index>`` ::=<NR1>.
        """
        return self._name

    @property
    def size(self) -> SlistSize:
        """Return the ``SLISt:SIZE`` command.

        Description:
            - This command returns the number sequences in sequence list.

        Usage:
            - Using the ``.query()`` method will send the ``SLISt:SIZE?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SIZE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SLISt:SIZE?
            ```
        """
        return self._size

    @property
    def subsequence(self) -> SlistSubsequence:
        """Return the ``SLISt:SUBSequence`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SLISt:SUBSequence?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SUBSequence?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.delete``: The ``SLISt:SUBSequence:DELete`` command.
            - ``.element``: The ``SLISt:SUBSequence:ELEMent[n]`` command tree.
            - ``.length``: The ``SLISt:SUBSequence:LENGth`` command.
            - ``.new``: The ``SLISt:SUBSequence:NEW`` command.
            - ``.tstamp``: The ``SLISt:SUBSequence:TSTamp`` command.
        """
        return self._subsequence



"""A class defining methods that VISA devices and control mixins must have."""

import logging

from abc import abstractmethod
from typing import Any, final, Tuple

from tm_devices.driver_mixins.device_control._abstract_device_control import (
    _AbstractDeviceControl,  # pyright: ignore[reportPrivateUsage]
)
from tm_devices.helpers import raise_failure, verify_values

_logger: logging.Logger = logging.getLogger(__name__)


class _AbstractDeviceVISAWriteQueryControl(_AbstractDeviceControl):  # pyright: ignore[reportUnusedClass]
    """Abstract class defining methods that VISA devices and control mixins must have."""

    @property
    def _no_error_string(self) -> str:
        """A string containing the expected error message when no error is present."""
        return ""

    @abstractmethod
    def query(self, *args: Any, **kwargs: Any) -> str:
        """Query the device."""

    @abstractmethod
    def write(self, *args: Any, **kwargs: Any) -> None:
        """Write to the device."""

    @final
    def expect_esr(
        self, esr: int, error_messages: Tuple[str, ...] = (), *, use_regex_match: bool = False
    ) -> bool:
        r"""Check for the expected error code and messages.

        Args:
            esr: Expected ``*ESR?`` value
            error_messages: Expected error buffer messages in a tuple.
            use_regex_match: A boolean indicating if the error messages should be matched
                using regular expressions.

        Returns:
            A boolean indicating if the check passed or failed, True means the check passed,
                False means the check failed (however, failing the check will always result in an
                AssertionError being raised, so the result will not really be usable).

        Raises:
            AssertionError: Indicating that the device's error code and messages don't match the
                expected values.
        """
        check_passed = True

        if not error_messages:
            error_messages = tuple(filter(None, (self._no_error_string,)))

        actual_esr, actual_error_messages = self._get_errors()

        # Compare the esr value
        try:
            verify_values(
                self.name_and_alias,
                esr,
                actual_esr,
                custom_message_prefix="expect_esr() error code check:",
                condense_printout=False,
            )
        except AssertionError as exc:
            check_passed &= False
            _logger.warning(exc)

        # Compare the error messages
        for expected_message, actual_message in zip(error_messages, actual_error_messages):
            try:
                verify_values(
                    self.name_and_alias,
                    expected_message,
                    actual_message,
                    use_regex_match=use_regex_match,
                    custom_message_prefix="expect_esr() error message check:",
                    condense_printout=False,
                )
            except AssertionError as exc:  # noqa: PERF203
                check_passed &= False
                _logger.warning(exc)

        if not check_passed:
            failure_message = (
                f"expect_esr() failed: error code {actual_esr!r} != {esr!r}; "
                f"error messages {actual_error_messages!r} != {error_messages!r}"
            )
            raise_failure(self.name_and_alias, failure_message, condense_printout=False)

        return check_passed



"""The cmdbatch commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CMDBatch {<NR1>OFF|ON}
    - CMDBatch?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Cmdbatch(SCPICmdWrite, SCPICmdRead):
    """The ``CMDBatch`` command.

    Description:
        - This command sets or queries the state of command batching. By batching commands, database
          transactions can be optimized, increasing command throughput. Also, batching allows for
          ALL commands in an individual batch to be order independent and accomplish the same result
          as if the commands were coupled. The Batch state is persistent and will be saved across
          power cycles, but will not be saved and recalled as part of a setup. In a setup scenario,
          the factory initial value is enabled.

    Usage:
        - Using the ``.query()`` method will send the ``CMDBatch?`` query.
        - Using the ``.verify(value)`` method will send the ``CMDBatch?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CMDBatch value`` command.

    SCPI Syntax:
        ```
        - CMDBatch {<NR1>OFF|ON}
        - CMDBatch?
        ```

    Info:
        - ``<NR1>`` = 0 turns command batching off; any other value turns command batching on.
        - ``OFF`` turns command batching off.
        - ``ON`` turns command batching on.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "CMDBatch") -> None:
        super().__init__(device, cmd_syntax)



"""The matharbflt commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MATHArbflt<x>:FILepath <QString>
    - MATHArbflt<x>:FILepath?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, ValidatedDynamicNumberCmd

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MatharbfltItemFilepath(SCPICmdWrite, SCPICmdRead):
    """The ``MATHArbflt<x>:FILepath`` command.

    Description:
        - This command or query sets the file path for a file of filter coefficients for the
          specified arbitrary filter. Setting a path will read that file and load the filter for
          ARBFLT<x>. Access these filters using a Math with an expression of the form 'ARBFlt<x>()'.

    Usage:
        - Using the ``.query()`` method will send the ``MATHArbflt<x>:FILepath?`` query.
        - Using the ``.verify(value)`` method will send the ``MATHArbflt<x>:FILepath?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MATHArbflt<x>:FILepath value`` command.

    SCPI Syntax:
        ```
        - MATHArbflt<x>:FILepath <QString>
        - MATHArbflt<x>:FILepath?
        ```

    Info:
        - ``<QString>`` specifies the path to the file of filter coefficients.
    """

    _WRAP_ARG_WITH_QUOTES = True


class MatharbfltItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``MATHArbflt<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MATHArbflt<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``MATHArbflt<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.filepath``: The ``MATHArbflt<x>:FILepath`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "MATHArbflt<x>"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._filepath = MatharbfltItemFilepath(device, f"{self._cmd_syntax}:FILepath")

    @property
    def filepath(self) -> MatharbfltItemFilepath:
        """Return the ``MATHArbflt<x>:FILepath`` command.

        Description:
            - This command or query sets the file path for a file of filter coefficients for the
              specified arbitrary filter. Setting a path will read that file and load the filter for
              ARBFLT<x>. Access these filters using a Math with an expression of the form
              'ARBFlt<x>()'.

        Usage:
            - Using the ``.query()`` method will send the ``MATHArbflt<x>:FILepath?`` query.
            - Using the ``.verify(value)`` method will send the ``MATHArbflt<x>:FILepath?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MATHArbflt<x>:FILepath value``
              command.

        SCPI Syntax:
            ```
            - MATHArbflt<x>:FILepath <QString>
            - MATHArbflt<x>:FILepath?
            ```

        Info:
            - ``<QString>`` specifies the path to the file of filter coefficients.
        """
        return self._filepath



# pyright: reportConstantRedefinition=none
"""The beeper commands module.

These commands are used in the following models:
SMU2601B_Pulse

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import BaseTSPCmd

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


# pylint: disable=too-few-public-methods
class Beeper(BaseTSPCmd):
    """The ``beeper`` command tree.

    Constants:
        - ``.OFF``: This command turns the beeper off.
        - ``.ON``: This command turns the beeper on.
    """

    OFF = "beeper.OFF"
    """str: This command turns the beeper off."""
    ON = "beeper.ON"
    """str: This command turns the beeper on."""

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "beeper") -> None:
        super().__init__(device, cmd_syntax)



"""Base Margin Tester device driver module."""

from __future__ import annotations

import os
import time

from abc import ABC, abstractmethod
from pathlib import Path
from typing import Any, Dict, Tuple, TYPE_CHECKING, Union

from requests.structures import CaseInsensitiveDict

from tm_devices.driver_mixins.device_control import RESTAPIControl
from tm_devices.drivers.device import Device, family_base_class
from tm_devices.helpers import DeviceConfigEntry, DeviceTypes
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813

if TYPE_CHECKING:
    from collections.abc import Mapping, MutableMapping

    from packaging.version import Version


@family_base_class
class MarginTester(Device, RESTAPIControl, ABC):
    """Base Margin Tester device driver."""

    ################################################################################################
    # Magic Methods
    ################################################################################################
    def __init__(self, config_entry: DeviceConfigEntry, verbose: bool) -> None:
        """Create a TMT4 device.

        Args:
            config_entry: A config entry object parsed by the DMConfigParser.
            verbose: A boolean indicating if verbose output should be printed. If True,
                communication printouts will be logged with a level of INFO. If False,
                communication printouts will be logged with a level of DEBUG.
        """
        super().__init__(config_entry, verbose)

        self._auth_token_file_path: Path = Path()

    ################################################################################################
    # Abstract Cached Properties
    ################################################################################################
    @cached_property
    @abstractmethod
    def adapter(self) -> str:
        """Return the device's connected adapter."""

    @cached_property
    @abstractmethod
    def fpga_version(self) -> Version:
        """Return the fpga version of the device."""

    @cached_property
    @abstractmethod
    def fw_version(self) -> Version:
        """Return the firmware version of the device."""

    @cached_property
    @abstractmethod
    def supported_technologies(self) -> Tuple[str, ...]:
        """Return the device's supported technologies."""

    ################################################################################################
    # Abstract Methods
    ################################################################################################
    @abstractmethod
    def request_about_info(self, verbose: bool = True) -> Dict[str, Any]:
        """Make a get request to 'about' and 'version' endpoints and return device information.

        Args:
            verbose: A boolean indicating if verbose output should be printed.
        """

    @abstractmethod
    def wait_till_unlocked(self, timeout: int = 30) -> None:
        """Wait until the device becomes unlocked.

        Args:
            timeout: The timeout to wait for an unlocked device in seconds.

        Raises:
            TimeoutError: If the device does not unlock within the timeout.
        """

    ################################################################################################
    # Properties
    ################################################################################################
    @cached_property
    def device_type(self) -> str:
        """Return a string representing the device type."""
        return DeviceTypes.MT.value

    @property
    def auth_token_file_path(self) -> str:
        """Return the path to the file containing the auth token."""
        return self._auth_token_file_path.as_posix()

    @auth_token_file_path.setter
    def auth_token_file_path(self, value: Union[str, os.PathLike[str]]) -> None:
        """Set the path to the file containing the auth token."""
        self._auth_token_file_path = Path(value)

    ################################################################################################
    # Public Methods
    ################################################################################################

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _close(self) -> None:
        """Close this device and all its used resources and components."""
        # This currently isn't needed for Margin Tester devices.
        self._is_open = False

    def _generate_headers(self) -> Mapping[str, str]:
        """Generate headers using a user provided auth token file.

        Returns:
            The header dictionary.

        Raises:
            AssertionError: Indicates that no auth token file is available.
        """
        if not (
            self._auth_token_file_path
            and self._auth_token_file_path.exists()
            and self._auth_token_file_path.is_file()
        ):
            msg = (
                "No auth token file set! Please set the ``.auth_token_file_path`` attribute "
                "to point to a file with an authorization token."
            )
            raise AssertionError(msg)
        headers: MutableMapping[str, str] = CaseInsensitiveDict()
        headers["Accept"] = "application/json"
        token = self._auth_token_file_path.read_text(encoding="utf-8").replace("\n", "")
        headers["Authorization"] = f"Bearer {token}"
        return headers

    def _get_errors(self) -> Tuple[int, Tuple[str, ...]]:
        """Get the current errors from the device.

        !!! note
            This method will clear out the error queue after reading the current errors.

        Returns:
            A tuple containing the current error code alongside a tuple of the current error
                messages.
        """
        return 0, ()

    def _open(self) -> bool:
        """Open necessary resources and components."""
        # Simply wait for the network connection, that indicates the device is open.
        time.sleep(15)
        self._is_open = self.wait_for_network_connection(
            120,
            accept_immediate_connection=bool(os.environ.get("TM_DEVICES_UNIT_TESTS_RUNNING")),
        )
        self._is_open &= self.wait_for_api_connection(
            120,
            accept_immediate_connection=True,
        )
        return self._is_open



"""The select commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SELect:CH<x> {ON|OFF|1|0}
    - SELect:CH<x>?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import DefaultDictPassKeyToFactory, SCPICmdRead, SCPICmdWrite, ValidatedChannel

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class SelectChannel(ValidatedChannel, SCPICmdWrite, SCPICmdRead):
    """The ``SELect:CH<x>`` command.

    Description:
        - Turns the display of the channel <x> waveform on or off, where <x > is the channel number.
          This command also resets the acquisition. The query returns whether the channel is on or
          off but does not indicate whether it is the specified waveform.

    Usage:
        - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

    SCPI Syntax:
        ```
        - SELect:CH<x> {ON|OFF|1|0}
        - SELect:CH<x>?
        ```

    Info:
        - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``OFF`` turns off the display of the specified waveform.
        - ``1`` turns on the display of the specified waveform. This waveform also becomes the
          selected waveform.
        - ``0`` turns off the display of the specified waveform.
    """


class Select(SCPICmdRead):
    """The ``SELect`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SELect?`` query.
        - Using the ``.verify(value)`` method will send the ``SELect?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.ch``: The ``SELect:CH<x>`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "SELect") -> None:
        super().__init__(device, cmd_syntax)
        self._ch: Dict[int, SelectChannel] = DefaultDictPassKeyToFactory(
            lambda x: SelectChannel(device, f"{self._cmd_syntax}:CH{x}")
        )

    @property
    def ch(self) -> Dict[int, SelectChannel]:
        """Return the ``SELect:CH<x>`` command.

        Description:
            - Turns the display of the channel <x> waveform on or off, where <x > is the channel
              number. This command also resets the acquisition. The query returns whether the
              channel is on or off but does not indicate whether it is the specified waveform.

        Usage:
            - Using the ``.query()`` method will send the ``SELect:CH<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``SELect:CH<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SELect:CH<x> value`` command.

        SCPI Syntax:
            ```
            - SELect:CH<x> {ON|OFF|1|0}
            - SELect:CH<x>?
            ```

        Info:
            - ``ON`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``OFF`` turns off the display of the specified waveform.
            - ``1`` turns on the display of the specified waveform. This waveform also becomes the
              selected waveform.
            - ``0`` turns off the display of the specified waveform.
        """
        return self._ch
